package com.hiramine.myexoplayerwithsurfaceview;

import androidx.appcompat.app.AppCompatActivity;

import android.net.Uri;
import android.os.Bundle;
import android.view.SurfaceView;
import android.view.ViewGroup;

import com.google.android.exoplayer2.Format;
import com.google.android.exoplayer2.MediaItem;
import com.google.android.exoplayer2.Player;
import com.google.android.exoplayer2.SimpleExoPlayer;

public class MainActivity extends AppCompatActivity
{
	private final String m_strUri = "https://commondatastorage.googleapis.com/gtv-videos-bucket/sample/BigBuckBunny.mp4";
	// 再生動画として、commondatastorage.googleapis.com の gtv-videos-bucket の sample に置いてある「Big Buck Bunny」動画ファイルを使用。
	// Big Buck Bunny
	// 　Copyright (C) 2008 Blender Foundation | peach.blender.org Some Rights Reserved.
	// 　Creative Commons Attribution 3.0 license. https://peach.blender.org/

	private SurfaceView     m_surfaceviewVideo;
	private SimpleExoPlayer m_simpleexoplayer;
	private boolean         m_bIsPlayingWhenActivityIsPaused;

	private final Player.Listener m_playerlistener = new Player.Listener()
	{
		@Override
		public void onIsPlayingChanged( boolean isPlaying )
		{
			if( isPlaying )
			{
				// 動画を再生するビューのサイズの調整
				adjustVideoViewSize();
			}
		}
	};

	@Override
	protected void onCreate( Bundle savedInstanceState )
	{
		super.onCreate( savedInstanceState );
		setContentView( R.layout.activity_main );

		// 動画を再生するビューの取得
		m_surfaceviewVideo = findViewById( R.id.surfaceview );

		// SimpleExoPlayerの作成と設定
		SimpleExoPlayer.Builder playerbuilder = new SimpleExoPlayer.Builder( this );
		m_simpleexoplayer = playerbuilder.build();
		m_simpleexoplayer.addListener( m_playerlistener );

		// 動画を再生するビューとSimpleExoPlayerの紐づけ
		m_simpleexoplayer.setVideoSurfaceView( m_surfaceviewVideo );

		// MediaItemの作成とSimpleExoPlayerへのセット
		Uri uriVideo = Uri.parse( m_strUri );
		m_simpleexoplayer.setMediaItem( MediaItem.fromUri( uriVideo ) );

		// 再生の準備
		m_simpleexoplayer.prepare();
		// 再生の開始
		m_simpleexoplayer.play();
	}

	// 初回表示時、および、ポーズからの復帰時
	@Override
	protected void onResume()
	{
		super.onResume();

		if( m_bIsPlayingWhenActivityIsPaused )
		{    // Activity pause時に動画再生中なら、動画再生再開
			m_simpleexoplayer.play();
		}
	}

	// アクティビティがフォアグラウンドからバックグラウンドに追いやられた時
	@Override
	protected void onPause()
	{
		// 動画再生中かどうかの取得と保持
		m_bIsPlayingWhenActivityIsPaused = m_simpleexoplayer.isPlaying();
		// 動画再生の一時停止
		m_simpleexoplayer.pause();

		super.onPause();
	}

	// 動画を再生するビューのサイズの調整
	private void adjustVideoViewSize()
	{
		// 動画があるかの確認
		Format videoformat = m_simpleexoplayer.getVideoFormat();
		if( null == videoformat )
		{
			return;
		}

		// 動画のサイズ
		int iVideoWidth  = videoformat.width;
		int iVideoHeight = videoformat.height;

		// 動画を再生するビューの親のサイズ
		ViewGroup viewgroupVideo = (ViewGroup)m_surfaceviewVideo.getParent();
		int iViewgroupWidth  = viewgroupVideo.getWidth();
		int iViewgroupHeight = viewgroupVideo.getHeight();

		// 動画の縦横比を保持しつつ、動画を再生するビューの親の中で最大限引きのばした、幅と高さを求める
		ViewGroup.LayoutParams layoutparams = m_surfaceviewVideo.getLayoutParams();
		double dVideoRatio     = (float)iVideoWidth / iVideoHeight;
		double dViewgropuRatio = (float)iViewgroupWidth / iViewgroupHeight;
		if( dVideoRatio > dViewgropuRatio )
		{
			layoutparams.width = iViewgroupWidth;
			layoutparams.height = (int)( (double)layoutparams.width / dVideoRatio );
		}
		else
		{
			layoutparams.height = iViewgroupHeight;
			layoutparams.width = (int)( (double)layoutparams.height * dVideoRatio );
		}

		// 求めた幅と高さを、動画を再生するビューに設定
		m_surfaceviewVideo.setLayoutParams( layoutparams );
	}
}